/*
  QtCurve (C) Craig Drummond, 2004 Craig.Drummond@lycos.co.uk


  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files(the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
*/

#include <qsettings.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qradiobutton.h>
#include <qgroupbox.h>
#include <kglobal.h>
#include <klocale.h>
#include "qtcurveconfig.h"
#include "../common.c"

extern "C"
{
    QWidget* allocate_kstyle_config(QWidget *parent)
    {
        KGlobal::locale()->insertCatalogue("kstyle_qtcurve_config");
        return new QtCurveConfig(parent);
    }
}

static const char *toStr(EDefBtnIndicator ind)
{
    switch(ind)
    {
        case IND_BORDER:
            return "border";
        case IND_NONE:
            return "none";
        default:
            return "corner";
    }
}

static const char *toStr(EGroove ind, bool allowNone=true)
{
    switch(ind)
    {
        case GROOVE_RAISED:
            return "raised";
        case GROOVE_NONE:
            if(allowNone)
                return "none";
        default:
            return "sunken";
    }
}

QtCurveConfig::QtCurveConfig(QWidget *parent)
             : QtCurveConfigBase(parent)
{
    QSettings s;

    border=s.readBoolEntry("/qtcurvestyle/Settings/border", true);
    borderWidget->setChecked(border);
    connect(borderWidget, SIGNAL(toggled(bool)), SLOT(borderToggled()));
    round=s.readBoolEntry("/qtcurvestyle/Settings/round", true);
    roundWidget->setChecked(round);
    connect(roundWidget, SIGNAL(toggled(bool)), SLOT(roundToggled()));

    bool etched=s.readBoolEntry("/qtcurvestyle/Settings/etched", true);
    QString sg=s.readEntry("/qtcurvestyle/Settings/sliderThumbs", QString::null);

    sliderThumbs=sg.isEmpty()
                     ? etched ? GROOVE_SUNKEN : GROOVE_RAISED
                     : qtc_to_groove(sg.latin1());
    sliderThumbsCombo->setCurrentItem(sliderThumbs);
    connect(sliderThumbsCombo, SIGNAL(activated(int)), SLOT(updateChanged()));

    handles=qtc_to_groove(s.readEntry("/qtcurvestyle/Settings/handles", DEF_HANDLE_STR).latin1());
    if(GROOVE_NONE==handles)
       handles=GROOVE_SUNKEN;
    handlesCombo->setCurrentItem(handles);
    connect(handlesCombo, SIGNAL(activated(int)), SLOT(updateChanged()));
    gradient=s.readBoolEntry("/qtcurvestyle/Settings/gradient", true);
    gradientWidget->setChecked(gradient);
    connect(gradientWidget, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    gradientPM=s.readBoolEntry("/qtcurvestyle/Settings/gradientPM", true);
    gradientPMWidget->setChecked(gradientPM);
    connect(gradientPMWidget, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    vArrow=s.readBoolEntry("/qtcurvestyle/Settings/vArrow", false);
    vArrowWidget->setChecked(vArrow);
    connect(vArrowWidget, SIGNAL(toggled(bool)), SLOT(updateChanged()));
    embolden=s.readBoolEntry("/qtcurvestyle/Settings/embolden", true);
    emboldenWidget->setChecked(embolden);
    connect(emboldenWidget, SIGNAL(toggled(bool)), SLOT(emboldenToggled()));
    crLabelHighlight=s.readBoolEntry("/qtcurvestyle/Settings/crLabelHighlight", true);
    crLabelHighlightWidget->setChecked(crLabelHighlight);
    connect(crLabelHighlightWidget, SIGNAL(toggled(bool)), SLOT(updateChanged()));

    defBtnIndicator=qtc_to_ind(s.readEntry("/qtcurvestyle/Settings/defBtnIndicator", DEF_IND_STR).latin1());
    defaultIndicatorCombo->setCurrentItem(defBtnIndicator);
    connect(defaultIndicatorCombo, SIGNAL(activated(int)), SLOT(dbiChanged()));

    versionWidget->setCurrentItem(
            !border && !round && GROOVE_RAISED==sliderThumbs && GROOVE_RAISED==handles && !gradient && !gradientPM &&
            vArrow && !embolden && crLabelHighlight && IND_BORDER==defBtnIndicator
                ? 0
                : border && !round && GROOVE_RAISED==sliderThumbs && GROOVE_RAISED==handles && !gradient && gradientPM &&
                  vArrow && !embolden && crLabelHighlight && IND_BORDER==defBtnIndicator
                      ? 1
                      : border && round && GROOVE_SUNKEN==sliderThumbs && GROOVE_SUNKEN==handles && gradient &&
                        gradientPM && !vArrow && embolden && !crLabelHighlight && IND_CORNER==defBtnIndicator
                            ? 2
                            : 3 );
    connect(versionWidget, SIGNAL(activated(int)), SLOT(versionChanged()));
    optionsFrame->setEnabled(versionWidget->currentItem()==3);
}

QtCurveConfig::~QtCurveConfig()
{
}


void QtCurveConfig::save()
{
    QSettings s;
    s.writeEntry("/qtcurvestyle/Settings/border", borderWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/round", roundWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/gradient", gradientWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/gradientPM", gradientPMWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/vArrow", vArrowWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/embolden", emboldenWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/crLabelHighlight", crLabelHighlightWidget->isChecked());
    s.writeEntry("/qtcurvestyle/Settings/defBtnIndicator", toStr((EDefBtnIndicator)defaultIndicatorCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/sliderThumbs", toStr((EGroove)sliderThumbsCombo->currentItem()));
    s.writeEntry("/qtcurvestyle/Settings/handles", toStr((EGroove)handlesCombo->currentItem(), false));
}

void QtCurveConfig::defaults()
{
    borderWidget->setChecked(true);
    roundWidget->setChecked(true);
    gradientWidget->setChecked(true);
    gradientPMWidget->setChecked(true);
    vArrowWidget->setChecked(false);
    emboldenWidget->setChecked(true);
    crLabelHighlightWidget->setChecked(false);
    defaultIndicatorCombo->setCurrentItem(IND_CORNER);
    sliderThumbsCombo->setCurrentItem(GROOVE_SUNKEN);
    handlesCombo->setCurrentItem(GROOVE_SUNKEN);
}

void QtCurveConfig::borderToggled()
{
    if(roundWidget->isChecked() && !borderWidget->isChecked())
        roundWidget->setChecked(false);
    updateChanged();
}

void QtCurveConfig::roundToggled()
{
    if(roundWidget->isChecked() && !borderWidget->isChecked())
        borderWidget->setChecked(true);
    updateChanged();
}

void QtCurveConfig::emboldenToggled()
{
    if(!emboldenWidget->isChecked() && IND_NONE==defaultIndicatorCombo->currentItem())
        defaultIndicatorCombo->setCurrentItem(IND_CORNER);
    updateChanged();
}

void QtCurveConfig::dbiChanged()
{
    if(IND_NONE==defaultIndicatorCombo->currentItem() && !emboldenWidget->isChecked())
        emboldenWidget->setChecked(true);
    updateChanged();
}

void QtCurveConfig::updateChanged()
{
    if ( borderWidget->isChecked()!=border ||
         roundWidget->isChecked()!=round ||
         gradientWidget->isChecked()!=gradient ||
         gradientPMWidget->isChecked()!=gradientPM ||
         vArrowWidget->isChecked()!=vArrow ||
         emboldenWidget->isChecked()!=embolden ||
         crLabelHighlightWidget->isChecked()!=crLabelHighlight ||
         defaultIndicatorCombo->currentItem()!=(int)defBtnIndicator ||
         sliderThumbsCombo->currentItem()!=(int)sliderThumbs ||
         handlesCombo->currentItem()!=(int)handles)
        emit changed(true);
    else
        emit changed(false);
}

void QtCurveConfig::versionChanged()
{
    int version=versionWidget->currentItem();

    optionsFrame->setEnabled(3==version);

    switch(version)
    {
        case 0:
            borderWidget->setChecked(false);
            roundWidget->setChecked(false);
            gradientWidget->setChecked(false);
            gradientPMWidget->setChecked(false);
            vArrowWidget->setChecked(true);
            emboldenWidget->setChecked(false);
            crLabelHighlightWidget->setChecked(true);
            defaultIndicatorCombo->setCurrentItem(IND_BORDER);
            sliderThumbsCombo->setCurrentItem(GROOVE_RAISED);
            handlesCombo->setCurrentItem(GROOVE_RAISED);
            break;
        case 1:
            borderWidget->setChecked(true);
            roundWidget->setChecked(false);
            gradientWidget->setChecked(false);
            gradientPMWidget->setChecked(true);
            vArrowWidget->setChecked(true);
            emboldenWidget->setChecked(false);
            crLabelHighlightWidget->setChecked(true);
            defaultIndicatorCombo->setCurrentItem(IND_BORDER);
            sliderThumbsCombo->setCurrentItem(GROOVE_RAISED);
            handlesCombo->setCurrentItem(GROOVE_RAISED);
            break;
        case 2:
            defaults();
            break;
        default:
            break;
    }
}

#include "qtcurveconfig.moc"
